@@ -317,7 +317,7 @@ REAL_SSH="$SSH"
 REAL_SSHD="$SSHD"
 SSH="$SSHLOGWRAP"
 
-if "$REAL_SSHD" -? 2>&1 | grep 'PKIX-SSH' > /dev/null; then
+if "$REAL_SSHD" -? 2>&1 | grep 'PKIX_SSH' > /dev/null; then
   sshd_type=pkix
 else
   sshd_type=other
@@ -621,9 +621,6 @@ for t in ${SSH_HOSTKEY_TYPES}; do
 	if test ! -f $OBJ/host.$t || test $OBJ/$t -nt $OBJ/host.$t ; then
 		# use key as host key, too
 		cp $OBJ/$t $OBJ/host.$t
-		case $t in
-		*xmss*) rm -f $OBJ/host.$t.*;; # remove state
-		esac
 	fi
 	echo HostKey $OBJ/host.$t >> $OBJ/sshd_config
 
