@@ -1,4 +1,4 @@
-/* 	$OpenBSD: tests.c,v 1.11 2025/04/15 04:00:42 djm Exp $ */
+/* 	$OpenBSD: tests.c,v 1.13 2025/09/04 00:34:17 djm Exp $ */
 /*
  * Regress test for misc helper functions.
  *
@@ -15,6 +15,8 @@ void test_argv(void);
 void test_strdelim(void);
 void test_hpdelim(void);
 void test_ptimeout(void);
+void test_xextendf(void);
+void test_misc(void);
 
 void
 tests(void)
@@ -26,6 +28,8 @@ tests(void)
 		test_strdelim();
 	test_hpdelim();
 	test_ptimeout();
+	test_xextendf();
+	test_misc();
 }
 
 void
