@@ -1,4 +1,4 @@
-/* $OpenBSD: serverloop.c,v 1.240 2024/06/17 08:28:31 djm Exp $ */
+/* $OpenBSD: serverloop.c,v 1.244 2025/09/25 06:23:19 jsg Exp $ */
 /*
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
  * Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
@@ -13,7 +13,7 @@
  *
  * SSH2 support by Markus Friedl.
  * Copyright (c) 2000, 2001 Markus Friedl.  All rights reserved.
- * Copyright (c) 2017-2023 Roumen Petrov.  All rights reserved.
+ * Copyright (c) 2017-2025 Roumen Petrov.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -40,10 +40,6 @@
 
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <sys/socket.h>
-#ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
-#endif
 
 #include <netinet/in.h>
 
@@ -51,13 +47,6 @@
 #include <fcntl.h>
 #include <pwd.h>
 #include <limits.h>
-#ifdef HAVE_POLL_H
-# include <poll.h>
-#else
-# ifdef HAVE_SYS_POLL_H
-#  include <sys/poll.h>
-# endif
-#endif
 #include <signal.h>
 #include <string.h>
 #include <termios.h>
@@ -114,6 +103,7 @@ extern int use_privsep;
 static int no_more_sessions = 0; /* Disallow further sessions. */
 
 static volatile sig_atomic_t child_terminated = 0;	/* The child has terminated. */
+static volatile sig_atomic_t request_channel_report_open = 0;
 
 /* Cleanup on signals (!use_privsep case only) */
 static volatile sig_atomic_t received_sigterm = 0;
@@ -142,6 +132,22 @@ sigchld_handler(int sig)
 	child_terminated = 1;
 }
 
+static void
+sigusr1_handler(int sig)
+{
+	UNUSED(sig);
+	request_channel_report_open = 1;
+}
+
+#ifdef SIGINFO
+static void
+siginfo_handler(int sig)
+{
+	UNUSED(sig);
+	request_channel_report_open = 1;
+}
+#endif
+
 static void
 sigterm_handler(int sig)
 {
@@ -401,9 +407,18 @@ server_loop2(struct ssh *ssh, Authctxt *authctxt)
 	UNUSED(authctxt);
 	debug("Entering interactive session for SSH2.");
 
-	if (sigemptyset(&bsigset) == -1 || sigaddset(&bsigset, SIGCHLD) == -1)
+	if (sigemptyset(&bsigset) == -1 ||
+	    sigaddset(&bsigset, SIGCHLD) == -1)
 		error_f("bsigset setup: %s", strerror(errno));
 	ssh_signal(SIGCHLD, sigchld_handler);
+	if (sigaddset(&bsigset, SIGUSR1) == -1)
+		error_f("bsigset setup: %s", strerror(errno));
+	ssh_signal(SIGUSR1, sigusr1_handler);
+#ifdef SIGINFO
+	if (sigaddset(&bsigset, SIGINFO) == -1)
+		error_f("bsigset setup: %s", strerror(errno));
+	ssh_signal(SIGINFO, siginfo_handler);
+#endif
 	child_terminated = 0;
 
 	if (!use_privsep) {
@@ -431,6 +446,10 @@ server_loop2(struct ssh *ssh, Authctxt *authctxt)
 		if (sigprocmask(SIG_BLOCK, &bsigset, &osigset) == -1)
 			error_f("bsigset sigprocmask: %s", strerror(errno));
 		collect_children(ssh);
+		if (request_channel_report_open) {
+			request_channel_report_open = 0;
+			channel_report_open(ssh);
+		}
 		wait_until_can_do_something(ssh,
 		    &pfd, &npfd_alloc, &npfd_active, &osigset,
 		    &conn_in_ready, &conn_out_ready);
@@ -744,7 +763,7 @@ server_input_hostkeys_prove(struct ssh *ssh, struct sshbuf **respp)
 	int r, ndx, success = 0;
 	const u_char *blob;
 	const char *rsa_kexalg = NULL;
-	u_char *sig = 0;
+	u_char *sig = NULL;
 	size_t blen, slen;
 
 	if (ssh->kex->hostkey_alg != NULL) {
